summaryrefslogtreecommitdiff
path: root/src/copy.c
diff options
context:
space:
mode:
authorPádraig Brady <P@draigBrady.com>2016-01-12 12:39:43 +0000
committerPádraig Brady <P@draigBrady.com>2016-01-13 11:16:37 +0000
commitd506af44befdbeabb027322652d4842314ba07be (patch)
treedf6d8c47e2d25c9d7d49503975fd1e017aa0836c /src/copy.c
parent2370c64a10924046d382f5205af3fcdaf0269959 (diff)
downloadcoreutils-d506af44befdbeabb027322652d4842314ba07be.tar.xz
mv: fix data loss with repeated source dir and same destination
commit v8.23-31-g90aa291 failed to consider this case, where the previous rename has failed, thus causing the following to remove the specified directory: mv dir dir dir * src/copy.c (copy_internal): Assume this rename attempt has succeeded, as a previous failure will already have been handled, and we don't want to remove the source directory in this case. * tests/cp/duplicate-sources.sh: Consolidate this test file to... * tests/mv/dup-source.sh: ...here. Add test cases for same source and dest. * tests/local.mk: Remove the consolidated test. * NEWS: Mention the bug fix. Reported at https://bugzilla.redhat.com/1297464
Diffstat (limited to 'src/copy.c')
-rw-r--r--src/copy.c12
1 files changed, 8 insertions, 4 deletions
diff --git a/src/copy.c b/src/copy.c
index aeb366a79..db2ce7377 100644
--- a/src/copy.c
+++ b/src/copy.c
@@ -2278,10 +2278,14 @@ copy_internal (char const *src_name, char const *dst_name,
error (0, 0, _("warning: source directory %s "
"specified more than once"),
quoteaf (top_level_src_name));
- /* We only do backups in move mode and for non dirs,
- and in move mode this won't be the issue as the source will
- be missing for subsequent attempts.
- There we just warn and return here. */
+ /* In move mode, if a previous rename succeeded, then
+ we won't be in this path as the source is missing. If the
+ rename previously failed, then that has been handled, so
+ pretend this attempt succeeded so the source isn't removed. */
+ if (x->move_mode && rename_succeeded)
+ *rename_succeeded = true;
+ /* We only do backups in move mode, and for non directories.
+ So just ignore this repeated entry. */
return true;
}
else if (x->dereference == DEREF_ALWAYS